@@ -6,9 +6,12 @@ from typing import Dict, List, Optional, Tuple, Any, Set
 
 from IPy import IP
 
+from irrd.rpki.status import RPKIStatus
 from irrd.rpsl.parser_state import RPSLParserMessages
+from irrd.scopefilter.status import ScopeFilterStatus
 from irrd.utils.text import splitline_unicodesafe
 from .fields import RPSLTextField
+from ..conf import get_setting
 
 RPSL_ATTRIBUTE_TEXT_WIDTH = 16
 TypeRPSLObjectData = List[Tuple[str, str, List[str]]]
@@ -55,9 +58,18 @@ class RPSLObject(metaclass=RPSLObjectMeta):
     attrs_multiple: List[str] = []
     ip_first: IP = None
     ip_last: IP = None
-    asn_first: IP = None
-    asn_last: IP = None
+    asn_first: Optional[int] = None
+    asn_last: Optional[int] = None
+    prefix: IP = None  # Note: not saved to DB, used for performance
+    prefix_length: Optional[int] = None
+    rpki_status: RPKIStatus = RPKIStatus.not_found
+    scopefilter_status: ScopeFilterStatus = ScopeFilterStatus.in_scope
     default_source: Optional[str] = None  # noqa: E704 (flake8 bug)
+    # Whether this object has a relation to RPKI ROA data, and therefore RPKI
+    # checks should be performed in certain scenarios. Enabled for route/route6.
+    rpki_relevant = False
+    # Fields whose values are discarded during parsing
+    discarded_fields: List[str] = []
 
     _re_attr_name = re.compile(r'^[a-z0-9_-]+$')
 
@@ -145,10 +157,18 @@ class RPSLObject(metaclass=RPSLObjectMeta):
                     result.add(field_name)
         return result
 
-    def render_rpsl_text(self) -> str:
-        """Render the RPSL object as an RPSL string."""
+    def render_rpsl_text(self, last_modified: datetime.datetime=None) -> str:
+        """
+        Render the RPSL object as an RPSL string.
+        If last_modified is provided, removes existing last-modified:
+        attributes and adds a new one with that timestamp, if self.source()
+        is authoritative.
+        """
         output = ""
+        authoritative = get_setting(f'sources.{self.source()}.authoritative')
         for attr, value, continuation_chars in self._object_data:
+            if authoritative and last_modified and attr == 'last-modified':
+                continue
             attr_display = f'{attr}:'.ljust(RPSL_ATTRIBUTE_TEXT_WIDTH)
             value_lines = list(splitline_unicodesafe(value))
             if not value_lines:
@@ -157,8 +177,16 @@ class RPSLObject(metaclass=RPSLObjectMeta):
                 if idx == 0:
                     output += attr_display + line
                 else:
-                    output += continuation_chars[idx-1] + (RPSL_ATTRIBUTE_TEXT_WIDTH-1) * ' ' + line
+                    continuation_char = continuation_chars[idx - 1]
+                    # Override the continuation char for empty lines #298
+                    if not line:
+                        continuation_char = '+'
+                    output += continuation_char + (RPSL_ATTRIBUTE_TEXT_WIDTH - 1) * ' ' + line
                 output += '\n'
+        if authoritative and last_modified:
+            output += 'last-modified:'.ljust(RPSL_ATTRIBUTE_TEXT_WIDTH)
+            output += last_modified.replace(microsecond=0).isoformat().replace('+00:00', 'Z')
+            output += '\n'
         return output
 
     def generate_template(self):
@@ -210,7 +238,7 @@ class RPSLObject(metaclass=RPSLObjectMeta):
                 return
 
             if not line.startswith(continuation_chars):
-                if current_attr:
+                if current_attr and current_attr not in self.discarded_fields:
                     # Encountering a new attribute requires saving the previous attribute data first, if any,
                     # which can't be done earlier as line continuation means we can't know earlier whether
                     # the attribute is finished.
@@ -231,7 +259,7 @@ class RPSLObject(metaclass=RPSLObjectMeta):
                 # Whitespace between the continuation character and the start of the data is not significant.
                 current_value += '\n' + line[1:].strip()
                 current_continuation_chars += line[0]
-        if current_attr:
+        if current_attr and current_attr not in self.discarded_fields:
             self._object_data.append((current_attr, current_value, current_continuation_chars))
 
     def _validate_object(self) -> None:
@@ -242,7 +270,7 @@ class RPSLObject(metaclass=RPSLObjectMeta):
         self.parsed_data: Dict[str, Any[str, List]] = {}
         if not self.messages.errors():
             self._validate_attribute_counts()
-        self._parse_attribute_data()
+        self._parse_attribute_data(allow_invalid_metadata=bool(self.messages.errors()))
 
         if self.strict_validation and not self.messages.errors():
             self.clean()
@@ -280,7 +308,7 @@ class RPSLObject(metaclass=RPSLObjectMeta):
                         f'Primary key attribute "{attr_pk}" on object {self.rpsl_object_class} is missing'
                     )
 
-    def _parse_attribute_data(self) -> None:
+    def _parse_attribute_data(self, allow_invalid_metadata=False) -> None:
         """
         Clean the data stored in attributes.
 
@@ -288,6 +316,11 @@ class RPSLObject(metaclass=RPSLObjectMeta):
         as they need to be indexed. All parsed values (e.g. without comments) are
         stored in self.parsed_data - stored in upper case unless a field is marked
         case sensitive.
+
+        If allow_invald_metadata is set, the parser will accept invalid metadata
+        being stored, but still make a best effort to extract data. This should
+        only be used if it is already known the object is invalid, and will
+        never be stored.
         """
         for idx, (attr_name, value, continuation_chars) in enumerate(self._object_data):
             field = self.fields.get(attr_name)
@@ -336,11 +369,11 @@ class RPSLObject(metaclass=RPSLObjectMeta):
                     # Some fields provide additional metadata about the resources to
                     # which this object pertains.
                     if field.primary_key or field.lookup_key:
-                        for attr in 'ip_first', 'ip_last', 'asn_first', 'asn_last':
+                        for attr in 'ip_first', 'ip_last', 'asn_first', 'asn_last', 'prefix', 'prefix_length':
                             attr_value = getattr(parsed_value, attr, None)
                             if attr_value:
                                 existing_attr_value = getattr(self, attr, None)
-                                if existing_attr_value:  # pragma: no cover
+                                if existing_attr_value and not allow_invalid_metadata:  # pragma: no cover
                                     raise ValueError(f'Parsing of {parsed_value.value} reads {attr_value} for {attr},'
                                                      f'but value {existing_attr_value} is already set.')
                                 setattr(self, attr, attr_value)
@@ -398,45 +431,6 @@ class RPSLObject(metaclass=RPSLObjectMeta):
             self._object_data.insert(insert_idx, (attribute, new_value, []))
             insert_idx += 1
 
-    def overwrite_date_new_changed_attributes(self, existing_obj=None) -> None:
-        """
-        Overwrite the date in any newly added changed: attributes per #242.
-        Which changed: lines are new is determined by comparing to existing_obj,
-        which should be another RPSLObject, or None if all changed: lines
-        should be considered new.
-        """
-        parsed_values_to_overwrite = set(self.parsed_data['changed'])
-        if existing_obj:
-            parsed_values_to_overwrite -= set(existing_obj.parsed_data['changed'])
-
-        # As the value is already validated by RPSLChangedField,
-        # we can safely make assumptions on the format.
-        new_object_data = []
-        removed_values_with_comment: List[Tuple[int, str]] = []
-        for idx, (attr_name, attr_value, continuation_chars) in enumerate(self._object_data):
-            if attr_name == 'changed':
-                attr_value_clean = attr_value.split('#')[0].strip()
-                if attr_value_clean in parsed_values_to_overwrite:
-                    removed_values_with_comment.append((idx, attr_value))
-                    continue
-            new_object_data.append((attr_name, attr_value, continuation_chars))
-        self._object_data = new_object_data
-
-        current_date = datetime.datetime.now().strftime('%Y%m%d')
-        for idx, value in removed_values_with_comment:
-            try:
-                content, comment = map(str.strip, value.split('#'))
-            except ValueError:
-                content = value.strip()
-                comment = ''
-            email = content.split(' ')[0]  # Ignore existing date
-            if comment:
-                new_value = f'{email} {current_date} # {comment}'
-            else:
-                new_value = f'{email} {current_date}'
-            self._object_data.insert(idx, ('changed', new_value, []))
-            self.messages.info(f'Set date in changed line "{value}" to today.')
-
     def __repr__(self):
         source = self.parsed_data.get('source', '')
         return f'{self.rpsl_object_class}/{self.pk()}/{source}'
